/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */


/* IANA addr settings */

#ifndef INADDR_ALLHOSTS_GROUP 
#define INADDR_ALLHOSTS_GROUP          0xe0000001U    /* 224.0.0.1 */
#endif 

#ifndef INADDR_ALLRTRS_GROUP
#define INADDR_ALLRTRS_GROUP           0xe0000002U    /* 224.0.0.2 */
#endif

#ifndef INADDR_PIM_ALLRTRS_GROUP
#define INADDR_PIM_ALLRTRS_GROUP       0xe000000DU    /* 224.0.0.13 */
#endif

#ifndef INADDR_MAX_LOCAL_GROUP
#define INADDR_MAX_LOCAL_GROUP         0xe00000ffU    /* 224.0.0.255 */
#endif

#ifndef LOCAL_MCAST
#define LOCAL_MCAST(x)  (((x) & htonl(0xFFFFFF00)) == htonl(0xE0000000))
#endif

#define TIMER_INTERVAL		        5
#define HELLO_INTERVAL 			30 /* Default */
#define GROUP_QUERY_INTERVAL		60
#define JOIN_PRUNE_INTERVAL		60
#define DATA_TIMER_INTERVAL             20 /* We have */
#define REGISTER_STOP_TIMER 60


/* From PIM-spec pp 30 */
#define PIM_HELLO               0
#define PIM_REGISTER            1
#define PIM_REGISTER_STOP       2
#define PIM_JOIN_PRUNE          3
#define PIM_BOOTSTRAP		4
#define PIM_ASSERT              5
#define PIM_GRAFT               6
#define PIM_GRAFT_ACK           7
#define PIM_CANDIDATE_RP_ADVERTISEMENT 8
#define PIM_REGISTER_WHOLEPACKET_CRC   9

#define PIM_NO                        0
#define PIM_DENSE_MODE                1
#define PIM_DENSE_SLOW_START_MODE     2
#define PIM_SPARSE_MODE               3
#define PIM_SPARSE_DENSE_MODE         4

#define PIM_VERSION_NONE 0
#define PIM_VERSION_1 1
#define PIM_VERSION_2 2
#define PIMREG_IDX (MAXVIFS-1) /* Interface index for special device pimreg */

struct encoded_unicast {
  u_int8_t family;
  u_int8_t type ;
  u_int32_t addr;
};

struct encoded_group {
  u_int8_t family;
  u_int8_t type ;
  u_int8_t reserved;
  u_int8_t masklen ;
  u_int32_t addr;
};

struct encoded_source {
  u_int8_t family;
  u_int8_t type ;
#define RPT_BIT (1<<0)
#define WC_BIT  (1<<1)
#define S_BIT   (1<<2)
  u_int8_t bits;
  u_int8_t masklen ;
  u_int32_t addr;
};

struct jp_header {
  struct encoded_unicast neighbor;
  u_int8_t reserved;
  u_int8_t num_groups;
  u_int16_t holdtime;
};

struct jp_header_group {
  u_int16_t num_joins;
  u_int16_t num_prunes;
};

struct hello_option {
  u_int16_t type;
  u_int16_t length;
  u_int16_t value;
};

struct pimheader {
  u_int8_t version;
  u_int8_t reserved;
  u_int16_t checksum;
};

struct pimquery {
  u_int8_t mode;
  u_int8_t reserved;
  u_int16_t holdtime;
};

struct pimreg_stop {
  u_int32_t gaddr;
  u_int32_t saddr;
};

struct pim_reachability {
  u_int32_t gaddr;
  u_int32_t gmask;
  u_int32_t rpaddr;
  u_int16_t reserved;
  u_int16_t holdtime;
};

struct pim_assert {
  u_int32_t gaddr;
  u_int32_t gmask;
  u_int32_t saddr;
  u_int32_t preference;
  u_int32_t metric;
};

struct pim_bootstrap_header {
  u_int16_t fragment_tag;
  u_int8_t hash_mask_len;
  u_int8_t BSR_priority;
  struct encoded_unicast BSR_address;
};

struct pim_bootstrap_group_header {
  struct encoded_group group;
  u_int8_t RP_count;
  u_int8_t frag_RP_count;
  u_int16_t reserved;
};

struct pim_bootstrap_rp {
  struct encoded_unicast RP_address;
  u_int16_t holdtime;
  u_int8_t priority;
  u_int8_t reserved;
};

struct pim_neighbor {
  struct in_addr address;
  struct thread *t_hello_neighbor;
  int version;
  struct interface *ifp;
  time_t uptime;
  time_t exptime;

  struct zlog *log;                /* ZLOG stream to use for this neighbor -
                                   NULL means use main log */
};

struct pim_interface {
  struct thread *t_hello;

  /* dense|sparse|sparse-dense */
  int mode; 
  int version;
  int running;
  u_int32_t flags;

#define IF_DR                   (1<<0)
#define IF_IGMP_QUERIER         (1<<1)
#define IF_IGMP_V1              (1<<2)
#define IF_PREFERENCE_SET       (1<<3)
#define IF_METRIC_SET           (1<<4)

  u_int32_t hello_period;
  u_int8_t igmp_leave_process;
  u_int8_t hello_period_set;
  struct interface *ifp;
  struct prefix *address;
  u_int32_t register_ttl;

/* IGMP timers from RFC 2236 */
  u_int32_t Robustness_Variable;
  u_int32_t Query_Interval;
  u_int32_t Query_Response_Interval;
  u_int32_t Group_Membership_Interval;
  u_int32_t Other_Querier_Present_Interval;
  u_int32_t Startup_Query_Interval;
  u_int32_t Startup_Query_Count;
  u_int32_t Last_Member_Query_Interval;
  u_int32_t Last_Member_Query_Count;
  u_int32_t Unsolicited_Report_Interval;
  u_int32_t Version_1_Router_Present_Timeout;

  struct thread *t_query;
  struct thread *t_send_join_prune;
  struct thread *t_other_querier_present;
  struct thread *t_Version_1_Router_Present;
  struct thread *t_register_supression;

  u_int32_t igmp_version;
  u_int32_t igmp_version_conf;
  list      igmp_static_group_list;

  /* PIM timers */
  u_int32_t join_prune_query_interval;
  u_int32_t join_prune_holdtime;
  u_int32_t join_prune_supression_timeout;
  u_int32_t data_timeout;
  u_int32_t trigger_join_prune_delay;
  u_int32_t trigger_join_prune_active;

  u_int32_t register_supression;

  /* Statistics */
  u_int32_t hello_sent;
  u_int32_t recv_badpackets;

  /* Assert preferenc and metrics  */
  u_int32_t preference ;
  u_int32_t metric ;

};

struct rp_set {
  u_int32_t bsr;
  list rp_list;
};

#define list_free_ifempty( LST ) if( list_isempty( LST ) ) \
      { list_delete_all( LST ); LST = 0; }
